/**
 *	@file	a_vector_test.cpp
 *	@brief	a_vector.h ̃eXg
 *	@author	Masashi Kitamura
 */

#include "unitst.h"
#include <stdio.h>
#include <stdlib.h>
#include <cassert>

#undef assert
#define assert(x)		UNITST_CHECK(x)
#include "../a_vector.h"



using namespace std;



class A_Vector_Test {
public:
	static void run() {
		test_array();
		test_pushpop();
		test_insert();
	}

	static void test_array() {
		a_vector<int, 5> ary(5);

		UNITST_EQ(ary.size(), 5);
		UNITST_EQ(ary.max_size(), 5);
		UNITST_EQ(ary.capacity(), 5);
		UNITST_EQ(ary.empty()	, 0);

		UNITST_CHECK_ANY_THROW( (ary.at(5) = 5) );

		int n = 0;
		for (n = 0; n < 5; ++n)
			ary[n] = n;
		for (n = 0; n < 5; ++n)
			ary[n] = ary[n]+1;

		UNITST_EQ(ary.front(), 1);
		UNITST_EQ(ary.back(), 5);

		a_vector<int, 5>::iterator it;
		for (n = 1, it = ary.begin(); it != ary.end(); ++it, ++n)
			UNITST_EQ(*it, n);

		a_vector<int, 5>::reverse_iterator ri;
		for (n = 5, ri = ary.rbegin(); ri != ary.rend(); ++ri, --n)
			UNITST_EQ(*ri, n);

		a_vector<int, 5> ary2(ary);
		UNITST_EQ_MEM(&ary2[0], &ary[0], 5*(sizeof ary[0]) );
		UNITST_EQ(ary == ary2, true );
		UNITST_EQ(ary != ary2, false);
		UNITST_EQ(ary <  ary2, false);
		UNITST_EQ(ary <= ary2, true );
		UNITST_EQ(ary >  ary2, false);
		UNITST_EQ(ary >= ary2, true );

		a_vector<int, 5> ary3;
		ary3 = ary;
		UNITST_EQ_MEM(&ary3[0], &ary[0], 5*(sizeof ary[0]) );
		UNITST_EQ(ary3, ary);
		ary3[2] = 10;
		UNITST_EQ((ary == ary3), false);
		UNITST_EQ((ary != ary3), true );
		UNITST_EQ((ary <  ary3), true );
		UNITST_EQ((ary <= ary3), true );
		UNITST_EQ((ary >  ary3), false);
		UNITST_EQ((ary >= ary3), false);
		ary3[2] = 1;
		UNITST_EQ((ary == ary3), false);
		UNITST_EQ((ary != ary3), true );
		UNITST_EQ((ary <  ary3), false);
		UNITST_EQ((ary <= ary3), false);
		UNITST_EQ((ary >  ary3), true );
		UNITST_EQ((ary >= ary3), true );

		a_vector<int, 5> ary4 = ary3;

		ary2.swap(ary4);
		UNITST_EQ(ary2, ary3);
		UNITST_EQ(ary4, ary);

		swap(ary2, ary4);
		UNITST_EQ(ary2, ary);
		UNITST_EQ(ary4, ary3);

		a_vector<int, 10> ary5 = ary;
		UNITST_EQ_MEM(&ary5[0], &ary[0], 5*(sizeof ary[0]) );
		ary5.resize(10);

		for (n = 0; n < int(ary5.size()); ++n)
			ary5[n] = (n+1)*2;
		ary4.assign(ary5.begin(), ary5.begin()+ 5);
		// ary4 = ary5;
		UNITST_EQ_MEM(&ary4[0], &ary5[0], 5*(sizeof ary[0]) );

		ary4.assign(5U, (int)500);
		for (n = 0; n < 5; ++n)
			UNITST_EQ(ary4[n], 500);
	}


	static void test_pushpop() {
		a_vector<int, 5> v;
		int n;

		UNITST_EQ(v.size(), 0);
		UNITST_EQ(v.max_size(), 5);
		UNITST_EQ(v.capacity(), 5);
		UNITST_EQ(v.empty()   , 1);

		for (n = 0; n < 5; ++n) {
			v.push_back(n+1);
			UNITST_EQ( v[n] ,v.at(n) );
			UNITST_EQ(v.size(), unsigned(n+1));
			UNITST_EQ(v.max_size(), 5);
			UNITST_EQ(v.capacity(), 5);
			UNITST_EQ(v.empty()   , 0);
			UNITST_EQ(v.front(), 1);
			UNITST_EQ(v.back(), n+1);
		}
		UNITST_CHECK_ANY_THROW( (v.at(5) = 5) );

		for (n = 5; --n >= 5;) {
			UNITST_EQ(v.back(), n+1);
			v.pop_back();
		}

		v.clear();
		UNITST_EQ(v.size(), 0);
		UNITST_EQ(v.empty()   , 1);

		v.resize(3);
		UNITST_EQ(v.size(), 3);
		UNITST_EQ(v.empty()   , 0);

		v.assign(3U, 999);
		for (n = 0; n < 3; ++n)
			UNITST_EQ(v[n], 999);

	}

	static void test_insert() {
		a_vector<int, 5> v;

		v.insert( v.begin(), 1 );
		UNITST_EQ(v.size(), 1);
		UNITST_EQ(v.empty()   , 0);

		v.insert( v.end(), 5 );
		UNITST_EQ(v.size(), 2);
		UNITST_EQ(v.empty()   , 0);
		static const int m2[] = { 1, 5 };
		UNITST_EQ_MEM(&v[0], m2, sizeof m2);

		v.insert( v.begin()+1, 3 );
		UNITST_EQ(v.size(), 3);
		UNITST_EQ(v.empty()   , 0);
		static const int m3[] = { 1, 3, 5 };
		UNITST_EQ_MEM(&v[0], m3, sizeof m3);

		v.erase( v.begin()+1 );
		UNITST_EQ(v.size()	, 2);
		UNITST_EQ(v.empty() , 0);
		UNITST_EQ_MEM(&v[0], m2, sizeof m2);

		static const int mi[] = {2,3,4};
		v.insert( v.begin()+1, mi, mi+3 );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ(v.empty() , 0);
		static const int m5[] = { 1, 2, 3, 4, 5 };
		UNITST_EQ_MEM(&v[0], m5, sizeof m5);

		v.erase( v.begin() );
		UNITST_EQ(v.size()	, 4);
		UNITST_EQ(v.empty() , 0);
		static const int m4a[] = { 2, 3, 4, 5 };
		UNITST_EQ_MEM(&v[0], m4a, sizeof m4a);

		v.erase( v.begin()+1, v.end()-1 );
		UNITST_EQ(v.size()	, 2);
		UNITST_EQ(v.empty() , 0);
		static const int m2b[] = { 2, 5 };
		UNITST_EQ_MEM(&v[0], m2b, sizeof m2b);

		v.insert( v.end(), mi, mi+3 );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ(v.empty() , 0);
		v[0] = 1;
		static const int m5b[] = { 1, 5, 2, 3, 4, };
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);

		a_vector<int, 10> v2;
		v2.assign(v.begin(), v.end());
		UNITST_EQ(v2.size()  , 5);
		UNITST_EQ(v2.empty() , 0);
		UNITST_EQ_MEM(&v2[0], m5b, sizeof m5b);

		v2.insert(v2.end(), v2.begin(), v2.end());
		static const int m10b[] = { 1, 5, 2, 3, 4, 1, 5, 2, 3, 4, };
		UNITST_EQ_MEM(&v2[0], m10b, sizeof m10b);

		v.erase( v.end()-2, v.end() );
		UNITST_EQ(v.size()	, 3);
		UNITST_EQ(v.empty() , 0);
		static const int m3b[] = { 1, 5, 2, };
		UNITST_EQ_MEM(&v[0], m3b, sizeof m3b);

		v.erase( v.begin(), v.end() );
		UNITST_EQ(v.size()	, 0);
		UNITST_EQ(v.empty() , 1);

		v.insert( v.begin(), 5U, 123 );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ(v.empty() , 0);
		static const int m5c[] = { 123, 123, 123, 123, 123 };
		UNITST_EQ_MEM(&v[0], m5c, sizeof m5c);

		//x v = v2;
		v.assign(v2.begin(), v2.begin() + v.capacity());

		UNITST_CHECK_ANY_THROW( v.insert( (int*)NULL, 1 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);
		UNITST_CHECK_ANY_THROW( v.insert( (int*)-1, 1 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);
		UNITST_CHECK_ANY_THROW( v.insert( v.begin(), 1 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);

		UNITST_CHECK_ANY_THROW( v.insert( v.begin()-1, m5c, m5c+3 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);
		UNITST_CHECK_ANY_THROW( v.insert( v.end()+1, m5c, m5c+3 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);
		UNITST_CHECK_ANY_THROW( v.insert( v.end(), m5c, m5c+1 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);

		UNITST_PRINTF(("ȉ 4s̃G[bZ[Wΐ\n"));
		//UNITST_CHECK_ANY_THROW
		( v.erase( (int*)NULL ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);
		//UNITST_CHECK_ANY_THROW
		( v.erase( (int*)-1 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);

		//UNITST_CHECK_ANY_THROW
		( v.erase( v.begin()-1, v.end() ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);
		//UNITST_CHECK_ANY_THROW
		( v.erase( v.begin(), v.end()+1 ) );
		UNITST_EQ(v.size()	, 5);
		UNITST_EQ_MEM(&v[0], m5b, sizeof m5b);

		v.erase( v.end()-2, v.end() );
		UNITST_CHECK_ANY_THROW( v.insert( v.end(), m5b+5-2, m5b+5+1 ) );
		UNITST_EQ(v.size()	, 3);
		UNITST_EQ_MEM(&v[0], m5b, sizeof(int)*3);
	}

};


UNITST_TEST(a_vector_test) {
	A_Vector_Test::run();
}
